#!/usr/bin/env python
#-*- coding: utf-8 -*-

## Constantes
# Fichier d'où lire les comptes Unix
PASSWDF = 'passwd'
# Fichier d'où lire les groupse Unix
GRPF = 'group'
# Fichier d'où lire les hashs Samba
SMBPWDF = 'smbpasswd'
# Partie fixe du dossier utilisateur Unix
UNIXHOMEPREFIX = '/export/home/'
# Partie fixe du dossier utilisateur Samba
SMBHOMEPREFIX = '\\\\\\\\iluvatar.arda\\homes\\'
# Commande paramétrée de création de l'utilisateur
PWCMD = 'smbldap-useradd -a -u %(uid)s -g %(gid)s -c "%(name)s" -s %(shell)s \
-d %(unixhome)s -C "%(smbhomeprefix)s%(smbhome)s" %(login)s\n'
# Commande paramétrée de création du groupe
GPCMD = 'smbldap-groupadd -a -g %(gid)s %(group)s\n'
# Instructions LDAP paramétrées pour la définition des mots de passe
LDC = """dn: uid=%(login)s,ou=people,dc=arda
changetype: modify
replace: userPassword
userPassword: {crypt}%(unixhash)s
-
replace: sambaNTPassword
sambaNTPassword: %(nthash)s
-
replace: sambaPwdLastSet
sambaPwdLastSet: 1\n\n"""


def homedir_deprefix(homedir):
    """Retire la partie fixe (UNIXHOME) du chemin Unix d'un dossier 
    utilisateur et renvoie le résultat.
    Lève une exception en cas d'échec. Il faut donc traiter en amont les cas
    particuliers, notamment celui du '/dev/null'.
    Ce résultat pourra être facilement modifié pour donner le chemin Samba
    du dossier utilisateur.

    """
    prefixLen = len(UNIXHOMEPREFIX)
    assert homedir[:prefixLen] == UNIXHOMEPREFIX, "Format de dossier \
utilisateur imprévu : %s" % homedir
    return homedir[prefixLen:]

def smbpwd_parse(smblines):
    """Renvoie un dictionnaire donnant le hash LM pour chaque utilisateur.
    Prend en paramètre l'ensemble des lignes du SMBPWDF.
    Ce dico servira ensuite à donner le hash du mot de passe Samba au
    serveur LDAP.
    
    """
    d = dict()
    for line in smblines:
        line = line.split(':')
        d[line[0]] = line[3]
    return d

def pwd_parse(pwdline):
    """Renvoie un dictionnaire contenant les éléments lus d'une ligne
    du fichier PASSWDF.
    On incluera ensuite ces données dans la commande générée.
    
    """
    return unix_parse(pwdline, \
            ('login', 'unixhash', 'uid', 'gid', 'name', 'unixhome', 'shell'))

def grp_parse(grpline):
    """Renvoie un dictionnaire contenant les éléments lus d'une ligne
    du fichier GRPF.
    On incluera ensuite ces données dans la commandé générée.
    
    """
    return unix_parse(grpline, ('group', 'void', 'gid'))

def unix_parse(line, attrs):
    """Renvoie un dictionnaire contenant les éléments lus d'une ligne de
    PASSWDF ou GRPF.
    Une liste d'attributs est passée en paramètre.

    """
    d = dict()
    l = line.split(':')
    for (num, attr) in enumerate(attrs):
        d[attr] = l[num]
    return d
   
def make_smbhome(unixhome):
    """Renvoie un dictionnaire donnant les différentes valeurs concernant
    le dossier utilisateur Samba : smbhomeprefix et smbhome.
    Prend donc en paramètre le chemin Unix complet du dossier utilisateur.
    Le dico retourné pourra être fusionné avec celui passé en paramètre.

    """
    d = dict()
    # Cas où l'utilisateur n'a pas de dossier
    if unixhome == '/dev/null':
        d['smbhomeprefix'] = d['smbhome'] = ""
    else:
        d['smbhomeprefix'] = SMBHOMEPREFIX
        d['smbhome'] = homedir_deprefix(unixhome).replace('/', '\\')
    return d

def make_nthash(d, smbd):
    """Renvoie le hash NT de l'utilisateur tel qu'il devra être inséré dans
    la base LDAP.
    Prend en paramètre un dico disposant au moins des clés 'login' et
    'unixhash', ainsi que le dico des hashs Samba généré par smpbwd_parse().
    Le résultat renvoyé pourra être inséré dans d."""
    if d['login'] in smbd:
        return smbd[d['login']]
    else:
        return 'XXX'
    
if __name__ == '__main__':
    """Boucle principale. Protège des inclusions."""
    # On récupère les données des trois fichiers
    pwdl = open(PASSWDF).readlines()
    smbl = open(SMBPWDF).readlines()
    grpl = open(GRPF).readlines()
    # On parse le fichier de hashs Samba dans un dico propre
    smbd = smbpwd_parse(smbl)
    # On ouvre nos trois fichiers de sortie
    cmdf = open('./create_users', 'w')
    gpf = open('./create_groups', 'w')
    ldf = open('./set_passwords.ldif', 'w')
    # On traite ensuite chaque login indépendamment.
    for user in pwdl:
        # On parse la ligne Unix
        # Note : readlines() prend aussi le caractère de fin de ligne,
        # on le retire donc ici.
        d = pwd_parse(user[:-1])
        # On ajoute le mot de passe Samba
        d['nthash'] = make_nthash(d, smbd)
        # Et le dossier utilisateur Samba
        d.update(make_smbhome(d['unixhome']))
        # On écrit
        cmdf.write(PWCMD % d)
        ldf.write(LDC % d)
    # Puis on traite les groupes
    for group in grpl:
        # On parse la ligne Unix comme dans l'autre boucle
        d = grp_parse(group[:-1])
        gpf.write(GPCMD % d)
    cmdf.close()
    gpf.close()
    ldf.close()
